/*  $Id: ConsoleErrHandler.java 1220 2011-09-11 14:03:19Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2010 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

import java.io.InputStream;
import java.io.PrintStream;
import java.util.ResourceBundle;
import java.util.Scanner;

/**
 * Console error handler.
 * Text based error handler, using by default standard input and output to interact
 * with user : reporting errors and choices, reading user responses.
 * @version $Id: ConsoleErrHandler.java 1220 2011-09-11 14:03:19Z sdevaux $
 * @author Sebastien Devaux
 */
public class ConsoleErrHandler implements ErrorHandler {
    /**
     * Create new console error handler.
     * System.in and System.out are defaults IOs
     * @param m handler mode.
     */
    public ConsoleErrHandler(int m) {
        this(m,System.in,System.out);
    }
    /**
     * Create new console error handler.
     * @param m handler mode.
     * @param in user responses input
     */ 
    public ConsoleErrHandler(int m,InputStream in,PrintStream out) {
        _mode=m;
        _input=new Scanner(in);
        _output=out;
    }
    /// Handler.handle()
    public void handle(Error e) {
        _error=true;
        _output.println(e.getMessage());
    }
    /// ErrorHandler.check
    public boolean checkOnly() {
        boolean cr=!_error;
        if (_error && _mode!=ErrHandlerFactory.M_NONE) {
            ResourceBundle resMessages=App.get().getResource(Error.class.getName());
            int code=-1;
            do {
                _output.println(resMessages.getString("prompt."+_msgId[_mode]));
                String userResponse=_input.nextLine(); 
                if (_mode!=ErrHandlerFactory.M_CONTINUE) {
                    String template=resMessages.getString("resp."+_msgId[_mode]);
                    int index=template.indexOf("/"+userResponse.toUpperCase(App.get().getLocale())+":");
                    if (index>=0) {
                        code=template.charAt(index+userResponse.length()+2)-'0';
                    }        
                }
            } while (code<0&&_mode!=ErrHandlerFactory.M_CONTINUE);
            if (code>=0) {
                cr=code!=0;
            }
        }
        return cr;
    }
    
    public boolean checkAndReset() {
    	boolean res=checkOnly();
    	_error=false;
    	return res;
    }
    /** Error handling mode (user interactio proto).
        one of ErrHandlerFactory.M_* consts. */
    private int _mode=0;
    /** error flag. true when error handled since last check() */
    private boolean _error=false;
    /** input reader */
    private Scanner _input;
    /** stream for message output */
    private PrintStream _output;
    /** standard confirmation messages ids */
    private final static String[] _msgId={"none","continue","ignore","retry"};
}
